﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Xml;
using System.Xml.Linq;

namespace MetrologSW
{
    public static class Databaze
    {
        public static DataClasses1DataContext dc = new DataClasses1DataContext();

        //private static string connectionString = @"Data Source=(Local)\SQLEXPRESS;Initial Catalog=metrology_db;Integrated Security=True";
        private static string connectionString = System.Configuration.ConfigurationManager.ConnectionStrings["MetrologSW.Properties.Settings.metrology_dbConnectionString"].ConnectionString;

        /// <summary>
        /// Zkontroluje, zda je databáze vytvořená a dostupná.
        /// </summary>
        /// <returns>
        ///   <c>true</c> pokud je databáze přístupná; jinak, <c>false</c>.
        /// </returns>
        public static bool IsDatabaseCreate()
        {
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                try
                {
                    connection.Open();
                    return true;
                }
                catch (SqlException)
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Vytvoří databázi podle připojovacího řetězce.
        /// </summary>
        public static void CreateDatabase()
        {
            using (var connection = new SqlConnection(connectionString))
            {
                try
                {
                    connection.Open();
                    var command = connection.CreateCommand();
                    command.CommandText = "CREATE DATABASE metrology_db";
                    command.ExecuteNonQuery();

                    command.CommandText = "CREATE TABLE Evidence(Id int IDENTITY(1, 1) NOT NULL,EvCislo nchar(10) NOT NULL,InvCislo nchar(10) NULL,VyrCislo nchar(10) NULL,Vyrobce nvarchar(50) NULL,Prislusenstvi nvarchar(50) NULL,Cena float NULL,Nakup date NULL,Kategorie nchar(10) NULL,Meridlo int NULL,Uzivatel int NULL,Misto int NULL,Kalibrace int NULL,PRIMARY KEY CLUSTERED(Id ASC)WITH(PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ONPRIMARY) ONPRIMARY";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence  WITH CHECK ADD  CONSTRAINT FK_Evidence_Kalibrace FOREIGN KEY(Kalibrace) REFERENCES Kalibrace (Id)";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence CHECK CONSTRAINT FK_Evidence_Kalibrace";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence  WITH CHECK ADD  CONSTRAINT FK_Evidence_Meridlo FOREIGN KEY(Meridlo) REFERENCES Meridlo (Id)";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence CHECK CONSTRAINT FK_Evidence_Meridlo";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence  WITH CHECK ADD  CONSTRAINT FK_Evidence_Misto FOREIGN KEY(Misto) REFERENCES Misto (Id)";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence CHECK CONSTRAINT FK_Evidence_Misto";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence  WITH CHECK ADD  CONSTRAINT FK_Evidence_Uzivatel FOREIGN KEY(Uzivatel) REFERENCES Uzivatel (Id)";
                    command.ExecuteNonQuery();

                    command.CommandText = "ALTER TABLE Evidence CHECK CONSTRAINT FK_Evidence_Uzivatel";
                    command.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
        }

        /// <summary>
        /// Importuje databázi z CSV souboru.
        /// </summary>
        /// <param name="umistesniSouboru">Umístění souboru v počítači.</param>
        /// <exception cref="FileNotFoundException">Soubor se nepodařilo nalézt.</exception>
        public static void ImportCSV(string umistesniSouboru)
        {
            if (!File.Exists(umistesniSouboru))
            { throw new FileNotFoundException("Soubor se nepodařilo nalézt.", umistesniSouboru); }

            Mouse.OverrideCursor = Cursors.Wait;
            using (StreamReader s = new StreamReader(umistesniSouboru, Encoding.Default))
            {
                string line;

                while ((line = s.ReadLine()) != null)
                {
                    string[] part = line.Split(';');

                    if (part[0] == "EvCislo") continue;
                    else
                    {
                        GetEvidence(part);

                        dc.SubmitChanges();
                    }
                }
            }

            Mouse.OverrideCursor = Cursors.Arrow;
        }

        /// <summary>
        /// Exportortuje CSV soubor na uživatelem dané umístění.
        /// </summary>
        /// <param name="umistenisouboru">Umístění v počítači.</param>
        public static void ExportCSV(string umistenisouboru)
        {
            using (StreamWriter s = new StreamWriter(umistenisouboru, false, Encoding.Default))
            {
                {
                    string zahlavi = "";
                    zahlavi += "EvCislo;InvCislo;VyrCislo;Vyrobce;Prislusenstvi;Cena;Nakup;Kategorie;Obor;Meridlo.Nazev;Meridlo.RozsahMin;Meridlo.RozsahMax;Meridlo.Rozlisitelnost;Meridlo.Jednotky;Meridlo.KalPerioda;";
                    zahlavi += "Misto.Stredisko;Misto.Umísteni;";
                    zahlavi += "Uzivatel.Prijmeni;Uzivatel.Jmeno;Uzivatel.OsCislo;";
                    zahlavi += "Kalibrace.KalibList;Kalibrace.Datum;Kalibrace.Stav;";
                    zahlavi += "Kontakt.Spolecnost;Kontakt.Prijmeni;Kontakt.Jmeno;Kontakt.Email1;Kontakt.Email2;Kontakt.Telefon;Kontakt.Telefon2;Kontakt.Web";

                    s.WriteLine(zahlavi);
                }
                foreach (Evidence ev in dc.Evidence)
                {
                    string line = "";

                    {
                        line += ev.EvCislo == null ? "" : ev.EvCislo.ToString().Trim();
                        line += ";";
                        line += ev.InvCislo == null ? "" : ev.InvCislo.ToString().Trim();
                        line += ";";
                        line += ev.VyrCislo == null ? "" : ev.VyrCislo.ToString().Trim();
                        line += ";";
                        line += ev.Vyrobce == null ? "" : ev.Vyrobce.ToString().Trim();
                        line += ";";
                        line += ev.Prislusenstvi == null ? "" : ev.Prislusenstvi.ToString().Trim();
                        line += ";";
                        line += ev.Cena == null ? "" : ev.Cena.Value.ToString().Trim();
                        line += ";";
                        line += ev.Nakup == null ? "" : ev.Nakup.Value.ToShortDateString().Trim();
                        line += ";";
                        line += ev.Kategorie == null ? "" : ev.Kategorie.ToString().Trim();
                        line += ";";

                        line += ev.Meridlo1.Obor == null ? "" : ev.Meridlo1.Obor.ToString();
                        line += ";";
                        line += ev.Meridlo1.Nazev == null ? "" : ev.Meridlo1.Nazev.ToString();
                        line += ";";
                        line += ev.Meridlo1.RozsahMin == null ? "" : ev.Meridlo1.RozsahMin.Value.ToString();
                        line += ";";
                        line += ev.Meridlo1.RozsahMax == null ? "" : ev.Meridlo1.RozsahMax.Value.ToString();
                        line += ";";
                        line += ev.Meridlo1.Rozlisitelnost == null ? "" : ev.Meridlo1.Rozlisitelnost.Value.ToString();
                        line += ";";
                        line += ev.Meridlo1.Jednotky == null ? "" : ev.Meridlo1.Jednotky.ToString();
                        line += ";";
                        line += ev.Meridlo1.KalPerioda == null ? "" : ev.Meridlo1.KalPerioda.Value.ToString();
                        line += ";";

                        line += ev.Misto1.Stredisko == null ? "" : ev.Misto1.Stredisko.ToString();
                        line += ";";
                        line += ev.Misto1.Umísteni == null ? "" : ev.Misto1.Umísteni.ToString();
                        line += ";";

                        line += ev.Uzivatel1.Prijmeni == null ? "" : ev.Uzivatel1.Prijmeni.ToString();
                        line += ";";
                        line += ev.Uzivatel1.Jmeno == null ? "" : ev.Uzivatel1.Jmeno.ToString();
                        line += ";";
                        line += ev.Uzivatel1.OsCislo == null ? "" : ev.Uzivatel1.OsCislo.ToString().Trim();
                        line += ";";

                        line += ev.Kalibrace1.KalibList == null ? "" : ev.Kalibrace1.KalibList.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Datum == null ? "" : ev.Kalibrace1.Datum.Value.ToShortDateString();
                        line += ";";
                        line += ev.Kalibrace1.Stav == null ? "" : ev.Kalibrace1.Stav.ToString();
                        line += ";";

                        line += ev.Kalibrace1.Kontakt.Spolecnost == null ? "" : ev.Kalibrace1.Kontakt.Spolecnost.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Prijmeni == null ? "" : ev.Kalibrace1.Kontakt.Prijmeni.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Jmeno == null? "": ev.Kalibrace1.Kontakt.Jmeno.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Email1 == null ? "" : ev.Kalibrace1.Kontakt.Email1.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Email2 == null ? "" : ev.Kalibrace1.Kontakt.Email2.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Telefon == null ? "" : ev.Kalibrace1.Kontakt.Telefon.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Telefon2 == null ? "" : ev.Kalibrace1.Kontakt.Telefon2.ToString();
                        line += ";";
                        line += ev.Kalibrace1.Kontakt.Web == null ? "" : ev.Kalibrace1.Kontakt.Web.ToString();
                    }

                    s.WriteLine(line);
                }
            }
        }

        #region Zpracovani importu      

        private static int GetRowsCount(string path)
        {
            int count = 0;

            using(StreamReader s = new StreamReader(path, Encoding.Default))
            {
                while (s.ReadLine() != null)
                    count++;
            }
            return count;
        }
          
        /// <summary>
        /// Získá data evidence z řádky souboru.
        /// </summary>
        /// <param name="line">Řádek záznamu.</param>
        private static void GetEvidence(string[] line)
        {
            var query = dc.Evidence.Where(x => x.EvCislo == line[0]);

            if (query.Count() == 0)
            {
                Evidence ev = new Evidence();

                double pom = 0;
                DateTime pomDate;

                ev.EvCislo = line[0];
                ev.InvCislo = line[1];
                ev.VyrCislo = line[2];
                ev.Vyrobce = line[3];
                ev.Prislusenstvi = line[4];
                ev.Cena = double.TryParse(line[5], out pom) ? pom : 0.0;
                ev.Nakup = DateTime.TryParse(line[6], out pomDate) ? pomDate : DateTime.MaxValue;
                ev.Kategorie = line[7];

                ev.Meridlo = GetMeridlo(line);
                ev.Uzivatel = GetUzivatel(line);
                ev.Misto = GetMisto(line);
                ev.Kalibrace = GetKalibrace(line);

                dc.Evidence.InsertOnSubmit(ev);
                dc.SubmitChanges();
            }
        }

        /// <summary>
        /// Získá data užvatele z řádky souboru.
        /// </summary>
        /// <param name="line">Řádek záznamu.</param>
        private static int GetUzivatel(string[] line)
        {
            var query = dc.Uzivatel.Where(x => x.OsCislo == line[19].Trim() && x.Prijmeni == line[17].Trim() && x.Jmeno == line[18].Trim());

            if (query.Count() == 0)
            {
                Uzivatel uz = new Uzivatel();

                uz.Prijmeni = line[17].Trim();
                uz.Jmeno = line[18].Trim();
                uz.OsCislo = line[19];

                dc.Uzivatel.InsertOnSubmit(uz);
                dc.SubmitChanges();

                return uz.Id;
            }
            else
                return query.First().Id;
            
        }

        /// <summary>
        /// Získá data měřidla z řádky souboru.
        /// </summary>
        /// <param name="line">Řádek záznamu.</param>
        private static int GetMeridlo(string[] line)
        {
            Meridlo me = new Meridlo();
            double pomInt;

            me.Obor = line[8];
            me.Nazev = line[9];
            me.RozsahMin = double.TryParse(line[10], out pomInt) ? pomInt : 0;
            me.RozsahMax = double.TryParse(line[11], out pomInt) ? pomInt : 0;
            me.Rozlisitelnost = double.TryParse(line[12], out pomInt) ? pomInt : 0;
            me.Jednotky = line[13];
            me.KalPerioda = double.TryParse(line[14], out pomInt) ? pomInt : 0;

            var query = dc.Meridlo.Where(x => x.Obor == me.Obor && x.Nazev == me.Nazev && x.RozsahMin == me.RozsahMin && x.RozsahMax == me.RozsahMax && x.Rozlisitelnost == me.Rozlisitelnost && x.Jednotky == me.Jednotky && x.KalPerioda == me.KalPerioda);

            if (query.Count() == 0)
            {
                dc.Meridlo.InsertOnSubmit(me);
                dc.SubmitChanges();

                return me.Id;
            }
            else
                return query.First().Id;
            
        }

        /// <summary>
        /// Získá data místa z řádky souboru.
        /// </summary>
        /// <param name="line">Řádek záznamu.</param>
        private static int GetMisto(string[] line)
        {
            var query = dc.Misto.Where(x => x.Stredisko == line[15] && x.Umísteni == line[16]);

            if (query.Count() == 0)
            {
                Misto mi = new Misto();

                mi.Stredisko = line[15];
                mi.Umísteni = line[16];

                dc.Misto.InsertOnSubmit(mi);
                dc.SubmitChanges();

                return mi.Id;
            }
            else
                return query.First().Id;

        }

        /// <summary>
        /// Získá data kalibrace z řádky souboru.
        /// </summary>
        /// <param name="line">Řádek záznamu.</param>
        private static int GetKalibrace(string[] line)
        {
            Kalibrace ka = new Kalibrace();
            double pomD;
            DateTime pomT;

            ka.KalibList = line[20];
            ka.Datum = DateTime.TryParse(line[21], out pomT) ? pomT : DateTime.MaxValue;
            ka.Stav = double.TryParse(line[22], out pomD) ? pomD : 0;

            ka.Kalibroval = GetKontakt(line);

            var query = dc.Kalibrace.Where(x => x == ka);

            if (query.Count() == 0)
            {
                dc.Kalibrace.InsertOnSubmit(ka);
                dc.SubmitChanges();
                return ka.Id;
            }
            else
                return query.First().Id;
        }

        /// <summary>
        /// Získá data kontaktu z řádky souboru.
        /// </summary>
        /// <param name="line">Řádek záznamu.</param>
        private static int GetKontakt(string[] line)
        {
            Kontakt ko = new Kontakt();

            ko.Spolecnost = line[23];
            ko.Prijmeni = line[24];
            ko.Jmeno = line[25];
            ko.Email1  = line[26];
            ko.Email2 = line[27];
            ko.Telefon = line[28];
            ko.Telefon2 = line[29];
            ko.Web = line[30];

            var query = dc.Kontakt.Where(x => x.Spolecnost == ko.Spolecnost && x.Prijmeni == ko.Prijmeni && x.Jmeno == ko.Jmeno && x.Email1 == ko.Email1 && x.Email2 == ko.Email2 && x.Telefon == ko.Telefon && x.Telefon2 == ko.Telefon2 && x.Web == ko.Web);

            if (query.Count() == 0)
            {
                dc.Kontakt.InsertOnSubmit(ko);
                dc.SubmitChanges();
                return ko.Id;
            }
            else
                return query.First().Id;

        }

        #endregion
    }
}
