﻿using MetrologSW;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data.Linq;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MetrologSW
{
    /// <summary>
    /// Interakční logika pro Karta.xaml
    /// </summary>
    public partial class Karta : Window
    {
        DataClasses1DataContext dc;
        Evidence ev, oldEv;

        public Karta(object evidence, DataClasses1DataContext dataContext)
        {
            InitializeComponent();
            dc = dataContext;
            if (evidence != null)
            {
                if (evidence.GetType() != typeof(Evidence))
                {
                    throw new FormatException("Špatný formát vstupního souboru.\nPožadovaný je typu 'Evidence'.");
                }

                DataContext = evidence;
                ev = (Evidence)evidence;
                oldEv = new Evidence();
                oldEv = ev;
            }
            else
                ev = new Evidence();

            GetForm();
        }

        #region Vyplnění karty

        /// <summary>
        /// Načte data do formuláře
        /// </summary>
        private void GetForm()
        {
            GetMeridla();
            GetUzivatele();
            GetMisto();
            GetKalibroval();
            GetKonecKalibrace();
        }
           
        /// <summary>
        /// Vybere všechny evidované typy měřidel
        /// </summary>
        private void GetMeridla()
        {
            dc.Refresh(RefreshMode.OverwriteCurrentValues, dc.Meridlo);
            boxMeridlo.ItemsSource = dc.Meridlo.OrderBy(x => x.Nazev).ThenBy(x => x.RozsahMin).ThenBy(x => x.RozsahMax);

            if (boxMeridlo.Items.Count > 0)
            {
                boxMeridlo.SelectedItem = ev.Meridlo1;
            }
        }

        /// <summary>
        /// Vybere, kdo má měřidlo v užívání
        /// </summary>
        private void GetUzivatele()
        {
            dc.Refresh(RefreshMode.OverwriteCurrentValues, dc.Uzivatel);
            boxUzivatel.ItemsSource = dc.Uzivatel.OrderBy(x => x.Prijmeni).ThenBy(x => x.Jmeno);

            if (boxUzivatel.Items.Count > 0)
            {
                boxUzivatel.SelectedItem = ev.Uzivatel1;
            }
        }

        /// <summary>
        /// Vybere poslední umístění vybraného měřidla
        /// </summary>
        private void GetMisto()
        {
            dc.Refresh(RefreshMode.OverwriteCurrentValues, dc.Misto);
            boxMisto.ItemsSource = dc.Misto.OrderBy(x => x.Stredisko).ThenBy(x => x.Umísteni);

            if (boxMisto.Items.Count > 0)
            {
                boxMisto.SelectedItem = ev.Misto1; 
            }
        }

        /// <summary>
        /// Vybere, kdo provedl poslední kalibraci pro vybrané měřidlo
        /// </summary>
        private void GetKalibroval()
        {
            dc.Refresh(RefreshMode.OverwriteCurrentValues, dc.Kontakt);
            var query = dc.Kontakt.OrderBy(x => x.Spolecnost).ThenBy(x => x.Prijmeni).ThenBy(x => x.Jmeno);

            cbKalibroval.ItemsSource = query;

            if (cbKalibroval.Items.Count > 0)
            {
                if (ev.Kalibrace1 != null)
                {
                    var match = query.Where(x => x.Id == ev.Kalibrace1.Kontakt.Id);
                    cbKalibroval.SelectedItem = match.First();
                }
            }
        }

        /// <summary>
        /// Vybere datum konce kalibrace pro vybrané měřidlo
        /// </summary>
        private void GetKonecKalibrace()
        {
            dc.Refresh(RefreshMode.OverwriteCurrentValues, dc.Kalibrace);
            var query = dc.Kalibrace.Where(x => x.Id == ev.Kalibrace.Value);

            boxKonecKal.Text = ev.Kalibrace != null && query.Count() > 0 ? query.First().ToString() : "";
        }
        #endregion

        private void Save()
        {
            //MessageBox.Show("Dokončit rutinu ukládání do databáze!!!");

            double pomCislo;
            DateTime pomDate;
            
            ev.EvCislo = boxEvC.Text;
            ev.InvCislo = boxInvC.Text;
            ev.VyrCislo = boxVyrC.Text;
            ev.Vyrobce = boxVyrobce.Text;
            
            ev.Cena = double.TryParse(boxCena.Text, out pomCislo) ? pomCislo : 0.0D;
            ev.Prislusenstvi = boxPrisl.Text;
            ev.Nakup = boxPorizeni.SelectedDate;
            ev.Kategorie = boxKategorie.Text;

            ev.Meridlo1 = (Meridlo)boxMeridlo.SelectedItem;
            ev.Uzivatel1 = (Uzivatel)boxUzivatel.SelectedItem;
            ev.Misto1 = (Misto)boxMisto.SelectedItem;

            ev.Kalibrace1.KalibList = boxKalList.Text;
            ev.Kalibrace1.Datum = DateTime.TryParse(boxDatKal.Text, out pomDate) ? pomDate : (DateTime?)null;
            ev.Kalibrace1.Stav = pbStav.Value;
            ev.Kalibrace1.Kontakt = (Kontakt)cbKalibroval.SelectedItem;

            dc.SubmitChanges();

            DialogResult = true;
        }

        /// <summary>
        /// Zpracuje událost Click na tlačítku Uložit
        /// </summary>
        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            Zavrit(sender);
        }

        /// <summary>
        /// Před zavřením zobrazí dialog pro uložení změn ve formuláři
        /// </summary>
        private void Zavrit(object sender)
        {
            if (sender.GetType() == typeof(Button))
            {
                switch (((Button)sender).Name)
                {
                    case "btnSave":
                        if (MessageBox.Show("Přejete si uložit provedené změny?", "Upozornění", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                        {
                            Save();
                            Close();
                        }
                            break;
                    case "btnCancel":
                        if (MessageBox.Show("Opustit formulář bez uložení změn?", "Upozornění", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                        {
                            Close();
                        }
                            break;
                }
                
            }
        }

        /// <summary>
        /// Zpracuje událost Click na tlačítku Uložit
        /// </summary>
        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            Zavrit(sender);
        }

    #region Obrazovky

        private void btnPridatMeridlo_Click(object sender, RoutedEventArgs e)
        {
            OknoMeridlo oknoMeridlo = new OknoMeridlo(null, dc);
            oknoMeridlo.ShowDialog();
            GetForm();
        }

        private void btnPridatUzivatele_Click(object sender, RoutedEventArgs e)
        {
            OknoUzivatel oknoUzivatel = new OknoUzivatel(null, dc);
            oknoUzivatel.ShowDialog();
            GetForm();
        }

        private void btnPridatMisto_Click(object sender, RoutedEventArgs e)
        {
            OknoMisto oknoMisto = new OknoMisto(null, dc);
            oknoMisto.ShowDialog();
            GetForm();
        }

        private void btnPridatKalibroval_Click(object sender, RoutedEventArgs e)
        {
            OknoKontakt o = new OknoKontakt(null, dc);
            o.ShowDialog();
            GetForm();
        }

        private void btnKalibrovat_Click(object sender, RoutedEventArgs e)
        {
            OknoKalibrace o = new OknoKalibrace(ev.Kalibrace1, dc, ev);
            o.ShowDialog();
            GetForm();
        }

        private void btnUpravitMeridlo_Click(object sender, RoutedEventArgs e)
        {
            OknoMeridlo o = new OknoMeridlo((Meridlo)boxMeridlo.SelectedItem, dc);
            o.ShowDialog();
            GetForm();
        }

        private void btnUpravitUzivatele_Click(object sender, RoutedEventArgs e)
        {
            OknoUzivatel o = new OknoUzivatel((Uzivatel)boxUzivatel.SelectedItem, dc);
            o.ShowDialog();
        }

        private void btnUpravitMisto_Click(object sender, RoutedEventArgs e)
        {
            OknoMisto o = new OknoMisto((Misto)boxMisto.SelectedItem, dc);
            o.ShowDialog();
            GetForm();
        }

        private void btnUpravitKalibroval_Click(object sender, RoutedEventArgs e)
        {
            OknoKontakt o = new OknoKontakt((Kontakt)cbKalibroval.SelectedItem, dc);
            o.ShowDialog();
            GetForm();
        }

        private void boxKalibrace_LostFocus(object sender, RoutedEventArgs e)
        {

        }

    #endregion

        /// <summary>
        /// Ošetření znaku '%' kvůli automatické změně statusbaru
        /// </summary>
        private void boxStav_LostFocus(object sender, RoutedEventArgs e)
        {
            boxStav.Text = boxStav.Text != "" && boxStav.Text[boxStav.Text.Length - 1] == '%' ? boxStav.Text.Remove(boxStav.Text.Length - 1, 1) : boxStav.Text;
        }

    }
}
