﻿using Microsoft.Win32;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data.Linq;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace MetrologSW
{
    /// <summary>
    /// Interakční logika pro MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public DataClasses1DataContext dc;

        public MainWindow()
        {
            InitializeComponent();

            if (!Databaze.IsDatabaseCreate())
            {
                if (MessageBox.Show("Nepodařilo se nalézt databázi nebo SQL server podle spojovacího řetězce v App.config.\nPokusit se databázi vytvořit?", "Chyba databáze", MessageBoxButton.YesNo, MessageBoxImage.Exclamation) == MessageBoxResult.Yes)
                {
                    try
                    {
                        Databaze.CreateDatabase();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Nepodařilo se vytvořit databázi.\n Zkontrolujte připojení na server a jeho status.\n\nProgram bude ukončen! \n\nChyba: " + ex.Message, "Chyba", MessageBoxButton.OK, MessageBoxImage.Error);
                        Application.Current.Shutdown();
                    }
                }
            }
            else
            {
                try
                {
                    dc = Databaze.dc;
                    DataContext = dc;
                }
                catch (Exception x)
                {
                    MessageBox.Show("K databázi se nepodařilo připojit.\n" + x.Message, "Chyba", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        private void tabulka_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            try
            {
                OtevritOknoKarta(dc.Evidence.Where(x => x.Id == ((Evidence)tabulka.SelectedItem).Id).First());
            }
            catch (Exception x)
            {
                MessageBox.Show("Měřidlo se nepodařilo nalézt." + Environment.NewLine + x.Message, "Chyba", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void OtevritOknoKarta(Evidence evidence)
        {
            try
            {
                Karta ev = new Karta(evidence, dc);
                if (ev.ShowDialog().Value == true)
                {
                    dc.SubmitChanges();
                }

                //FillTable();
            }
            catch (Exception x)
            {
                MessageBox.Show("Nastala chyba:\n" + x.Message, "Chyba", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void OtevritOknoUzivatel()
        {
            try
            {
                OknoUzivatel okno = new OknoUzivatel(dc.Uzivatel.First(), dc);
                if (okno.ShowDialog().Value == true)
                {
                    dc.SubmitChanges();
                }
            }
            catch (Exception x)
            {
                MessageBox.Show("Nastala chyba:\n" + x.Message, "Chyba", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

    #region Položky menu

        private void _close_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show("Přejete si před ukončením aplikace uložit všechny neuložené změny?", "Ukončení", MessageBoxButton.OKCancel, MessageBoxImage.Information) == MessageBoxResult.OK)
            {
                dc.SubmitChanges();
                Close();
            }
            else
            {
                Close();
            }
        }

        private void _prosle_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Filtr.GetFilter(sender);
            }
            catch (Exception x)
            {
                MessageBox.Show($"Nepodařilo se provést filtrování.\n{x.Message}", "Chyba", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            finally
            {
                tabulka.ItemsSource = Filtr.oc;
            }
        }

        private void _uzivatel_Click(object sender, RoutedEventArgs e)
        {
            OknoUzivatel o = new OknoUzivatel(null, dc);
            o.ShowDialog();
        }

        private void _misto_Click(object sender, RoutedEventArgs e)
        {
            OknoMisto o = new OknoMisto(null, dc);
            o.ShowDialog();
        }

        private void _meridlo_Click(object sender, RoutedEventArgs e)
        {
            OknoMeridlo o = new OknoMeridlo(null, dc);
            o.ShowDialog();
        }

        private void _kalibrace_Click(object sender, RoutedEventArgs e)
        {
            Karta o = new Karta(null, dc);
            o.ShowDialog();
        }

        private void _kontakt_Click(object sender, RoutedEventArgs e)
        {
            OknoKontakt o = new OknoKontakt(null, dc);
            o.WindowStartupLocation = WindowStartupLocation.CenterOwner;
            o.ShowDialog();
        }

        private void _export_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog o = new SaveFileDialog();
            o.Filter = "CSV soubory|*.csv|Textové soubory|*.txt|Všechny soubory|*.*";
            if (o.ShowDialog() == true)
            {
                Databaze.ExportCSV(o.FileName);
            }
        }

        private void _import_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog o = new OpenFileDialog();
            o.Filter = "CSV soubory|*.csv|Textové soubory|*.txt|Všechny soubory|*.*";
            if (o.ShowDialog() == true)
            {
                Databaze.ImportCSV(o.FileName);
            }
                        
        }

        

    #endregion

    }
}
