﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MetrologSW
{
    /// <summary>
    /// Interakční logika pro OknoKontakt.xaml
    /// </summary>
    public partial class OknoKontakt : Window
    {
        private DataClasses1DataContext dc;
        private Kontakt kontakt;

        public OknoKontakt(Kontakt kontakt, DataClasses1DataContext dataContext)
        {
            InitializeComponent();

            try
            {
                DataContext = dataContext.Kontakt;
                dc = dataContext;

                if (kontakt != null)
                {
                    cbSpolečnost.IsEnabled = true;
                    this.kontakt = kontakt;
                }
                else
                {
                    cbSpolečnost.IsEnabled = false;
                    this.kontakt = null;
                }
            }
            catch
            {

                throw new Exception("Nepodařilo se načíst data.");
            }

            GetKontakt();
        }

        private void GetKontakt()
        {
            cbSpolečnost.ItemsSource = dc.Kontakt.Where(x => x.Spolecnost != "").OrderBy(x => x.Spolecnost);
            if(kontakt != null) { cbSpolečnost.SelectedItem = kontakt; }
        }

        #region Events
        private void bOk_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult question = MessageBox.Show("Uložit kontakt?", "Uložit", MessageBoxButton.YesNoCancel, MessageBoxImage.Question);

            if (question == MessageBoxResult.Yes)
            {
                if (Validace())
                {
                    if (cbSpolečnost.SelectedItem == null)
                    {
                        kontakt = new Kontakt() { Spolecnost = boxSpolecnost.Text, Prijmeni = boxPrijmeni.Text, Jmeno = boxJmeno.Text, Email1 = boxEmail1.Text, Email2 = boxEmail2.Text, Telefon = boxTelefon1.Text, Telefon2 = boxTelefon2.Text, Web = boxWeb.Text };
                        dc.Kontakt.InsertOnSubmit(kontakt);
                    }
                    
                    dc.SubmitChanges();
                    Close();
                }
                else
                    MessageBox.Show("Zadány nedostatečné údaje.\nJméno společnosti je povinný údaj.", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
            }
            else if(question == MessageBoxResult.No)
                Close();
        }

        private void bCancel_Click(object sender, RoutedEventArgs e)
        {
            if(MessageBox.Show("Zahodit změny?", "Upozornění", MessageBoxButton.YesNo, MessageBoxImage.Exclamation) == MessageBoxResult.Yes)
            {
                Close();
            }
        }

        #endregion

        private bool Validace()
        {
            return boxSpolecnost.Text != "";
        }

        private void cbSpolečnost_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }
    }
}
