﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MetrologSW
{
    /// <summary>
    /// Interakční logika pro OknoMeridlo.xaml
    /// </summary>
    public partial class OknoMeridlo : Window
    {
        Meridlo m;
        DataClasses1DataContext dc;

        public OknoMeridlo(Meridlo meridlo, DataClasses1DataContext dataContext)
        {
            InitializeComponent();

            m = meridlo;
            dc = dataContext;

            GetMeridla();
        }

        private void btnOk_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult question = MessageBox.Show("Chcete uložit měřidlo?", "Upozornění", MessageBoxButton.YesNoCancel, MessageBoxImage.Question);

            if(question == MessageBoxResult.Yes)
            {
                if (m == null)
                {
                    Meridlo nove = new Meridlo() { Nazev = boxNazev.Text.Trim(), Obor = boxObor.Text.Trim(), Jednotky = boxJednotky.Text.Trim(), Rozlisitelnost = ConvertStringToDouble(boxRozlisitelnost.Text), RozsahMin = ConvertStringToDouble(boxRozsahMin.Text), RozsahMax = ConvertStringToDouble(boxRozsahMax.Text), KalPerioda = ConvertStringToDouble(boxPerioda.Text) };
                    dc.Meridlo.InsertOnSubmit(nove);
                }
                else
                {
                    m.KalPerioda = ConvertStringToDouble(boxPerioda.Text);
                    m.RozsahMin = ConvertStringToDouble(boxRozsahMin.Text);
                    m.RozsahMax = ConvertStringToDouble(boxRozsahMax.Text);
                    m.Rozlisitelnost = ConvertStringToDouble(boxRozlisitelnost.Text);
                }
                dc.SubmitChanges();
                Close();
            }
            else if(question == MessageBoxResult.No)
                Close();
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show("Ukončit bez uložení?", "Upozornění", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                Close();
            }
        }

        private void GetMeridla()
        {
            if (m == null)
            {
                cbMeridlo.IsEnabled = false;
            }
            else
            {
                cbMeridlo.IsEnabled = true;
                cbMeridlo.ItemsSource = dc.Meridlo.OrderBy(x => x.Nazev).ThenBy(x => x.RozsahMin).ThenBy(x => x.RozsahMax);
                cbMeridlo.SelectedItem = m;
            }
        }

        private double ConvertStringToDouble(string s)
        {
            double res;
            return double.TryParse(s, out res) ? res : 0.0;
        }
    }
}
